#include <FS.h>                   //this needs to be first, or it all crashes and burns...
#include <ArduinoJson.h>
#include <ESP8266WiFi.h>
#include <ESP8266WebServer.h>     ///https://github.com/esp8266/Arduino/tree/master/libraries/ESP8266WebServer
#include <DNSServer.h>
#include <WiFiManager.h>          //https://github.com/tzapu/WiFiManager
#include <Wire.h>
#include <LiquidCrystal_I2C.h>


// set the LCD address to 0x27 for a 16 chars and 2 line display
LiquidCrystal_I2C lcd(0x3f, 20, 4);


char char_server_emoncms[40] = "serveremoncms.com";
char char_port_emoncms[4] = "80";
char char_apikey_emoncms[33] = "****************************";

unsigned long lastConnectionTime = 0;             // last time you connected to the server, in milliseconds
const unsigned long postingInterval = 10L * 1000L;

const char* host;
int port;
const char* apikey;
const char* inputname1 = "realPower1";                    /// name input in emoncms
const char* inputname2 = "realPower2";
const char* inputname3 = "realPower3";
const char* inputname4 = "Irms1";
const char* inputname5 = "Irms2";
const char* inputname6 = "Irms3";
const char* inputname7 = "Vrms";
const char* nodedata   = "EmonTX";

// SSID WifiManager configuration network //

const char* ssid_apmode = "ESP_emoncms";

//flag for saving data
bool shouldSaveConfig = false;
bool initialConfig = false;
bool parameterOK = false;

#define AMUXSEL0 14     // AMUX Selector 0 if edited here edid in emonlib.cpp
#define AMUXSEL1 12     // AMUX Selector 1 if edited here edid in emonlib.cpp
#define AMUXSEL2 15     // AMUX Selector 2 if edited here edid in emonlib.cpp
#define LEDpin  13      // set LED

#define FILTERSETTLETIME 5000                                           //  Time (ms) to allow the filters to settle before sending data


const int CT1 = 1;
const int CT2 = 1;                                                      // Set to 0 to disable
const int CT3 = 1;

float ct_1 = 0;
float ct_2 = 0;
float ct_3 = 0;
float irms_1 = 0;
float irms_2 = 0;
float irms_3 = 0;
float vrm_0 = 0;

#include "EmonLibesp.h"
EnergyMonitor ct1, ct2, ct3;                                            // Create  instances for each CT channel

// Note: Please update emonhub configuration guide on OEM wide packet structure change:
// https://github.com/openenergymonitor/emonhub/blob/emon-pi/configuration.md
typedef struct {
  int power1, power2, power3, irms1, irms2, irms3, Vrms;
} PayloadTX;      // create structure - a neat way of packaging data for RF comms
PayloadTX emontx;

boolean settled = false;

//callback notifying us of the need to save config
void saveConfigCallback () {
  Serial.println("Should save config");
  shouldSaveConfig = true;
}

void setup()
{
#define button_config  5 ///PIN D1 NodeMCU  /// Normally Open Button

  pinMode(button_config, INPUT);

  lcd.init();
  lcd.backlight();
  lcd.setCursor(2, 1);
  lcd.print("Initializing...");
  delay(5000);
  lcd.clear();
  lcd.setCursor(0, 1);
  lcd.print("emonTX Shield 3Phase");
  delay(5000);
  lcd.clear();


  pinMode(AMUXSEL0, OUTPUT);   // sets the digital pin as output
  pinMode(AMUXSEL1, OUTPUT);
  pinMode(AMUXSEL2, OUTPUT);
  pinMode(LEDpin, OUTPUT);
  pinMode(BUILTIN_LED, OUTPUT);     // Initialize the BUILTIN_LED pin as an output

  Serial.begin(115200);

  if (CT1) ct1.current(1, 66.606);                                     // Setup emonTX CT channel (ADC input, calibration)
  if (CT2) ct2.current(2, 60.606);                                     // Calibration factor = CT ratio / burden resistance
  if (CT3) ct3.current(3, 60.606);                                     // emonTx Shield Calibration factor = (100A / 0.05A) / 33 Ohms


  if (CT1) ct1.voltage(0, 144, 1.7);                                // ct.voltageTX(ADC input, calibration, phase_shift) - make sure to select correct calibration for AC-AC adapter  http://openenergymonitor.org/emon/modules/emontx/firmware/calibration. Default set for Ideal Power adapter
  if (CT2) ct2.voltage(0, 144, 1.7);                                // 268.97 for the UK adapter, 260 for the Euro and 130 for the US.
  if (CT3) ct3.voltage(0, 144, 1.7);



  digitalWrite(LEDpin, HIGH);    // Setup indicator LED
  wifiManager();
}

void loop()
{


  Lcd();

  if (CT1) {
    ct1.calcVI(20, 2000);                                                 // Calculate all. No.of crossings, time-out
    emontx.power1 = ct1.realPower;
    ct_1 = (emontx.power1);
    emontx.irms1 = ct1.Irms;
    irms_1 = (emontx.irms1);
  }

  emontx.Vrms = ct1.Vrms * 100; // AC Mains rms voltage
  vrm_0 = (emontx.Vrms);

  if (CT2) {
    ct2.calcVI(20, 2000);                                                 // Calculate all. No.of crossings, time-out
    emontx.power2 = ct2.realPower;
    ct_2 = (emontx.power2);
    emontx.irms1 = ct2.Irms;
    irms_2 = (emontx.irms2);
  }

  if (CT3) {
    ct3.calcVI(20, 2000);                                                 // Calculate all. No.of crossings, time-out
    emontx.power3 = ct3.realPower;
    ct_3 = (emontx.power3);
    emontx.irms3 = ct3.Irms;
    irms_3 = (emontx.irms3);
  }


  // because millis() returns to zero after 50 days !
  if (!settled && millis() > FILTERSETTLETIME) settled = true;


  if (settled)                                                            // send data only after filters have settled
  {
    if (millis() - lastConnectionTime > postingInterval) {
      httpRequest();
    }
  }
}
