void wifiManager()
{

  //clean FS, for testing
  //SPIFFS.format();

  //read configuration from FS json
  Serial.println("mounting FS...");

  if (SPIFFS.begin()) {
    Serial.println("mounted file system");
    if (SPIFFS.exists("/config.json")) {
      //file exists, reading and loading
      Serial.println("reading config file");
      File configFile = SPIFFS.open("/config.json", "r");
      if (configFile) {
        Serial.println("opened config file");
        size_t size = configFile.size();
        // Allocate a buffer to store contents of the file.
        std::unique_ptr<char[]> buf(new char[size]);

        configFile.readBytes(buf.get(), size);
        DynamicJsonBuffer jsonBuffer;
        JsonObject& json = jsonBuffer.parseObject(buf.get());
        //json.printTo(Serial);                                              /////debug json parameters
        if (json.success()) {
          Serial.println("\nparsed json");

          strcpy(char_server_emoncms, json["char_server_emoncms"]);
          strcpy(char_port_emoncms, json["char_port_emoncms"]);
          strcpy(char_apikey_emoncms, json["char_apikey_emoncms"]);



        } else {
          Serial.println("failed to load json config");
        }
      }
    }
  } else {
    Serial.println("failed to mount FS");
  }
  //end read

  ////////////////////////////////////////////
  ////////////////////////////////////////////
  ////////////////////////////////////////////


  // Push button Enables WifiManager configuration   /////////////////
  // is configuration portal requested?
  if ( digitalRead(button_config) == LOW ) {

    digitalWrite(LEDpin, LOW);



    // The extra parameters to be configured (can be either global or just in the setup)
    // After connecting, parameter.getValue() will get you the configured value
    // id/name placeholder/prompt default length

    ////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

    WiFiManagerParameter custom_server_emoncms    ("char_server_emoncms", "server emoncms", char_server_emoncms, 40, "title= \"Parameter 1: server or Host Emoncms example: 192.168.0.12 or domainemoncms.com\"");
    WiFiManagerParameter custom_port_emoncms      ("char_port_emoncms", "port emoncms", char_port_emoncms, 4,       "title= \"Parameter 2: Port 80 default \"   type=\"number\"  ,  min=\"10\" max=\"9999\"   ");
    WiFiManagerParameter custom_apikey_emoncms    ("char_apikey_emoncms", "apikey write", char_apikey_emoncms, 33,  " type= password  maxlength=\"32\"  title= \"Parameter:3 Apikey Write Emoncms  example: 4ccb0b337dd7fa4b71f4601758e7f469\"    ");


    WiFiManager wifiManager;


    /////////  set config save notify callback ////
    wifiManager.setSaveConfigCallback(saveConfigCallback);

    //add all your parameters here
    ////// Adds parameter fields to Wifimanager ////////////////////////////

    //wifiManager.setDebugOutput(false);

    WiFiManagerParameter custom_text0("<p><strong> ESP8266 Emoncms Configuration </p> </strong>");
    wifiManager.addParameter(&custom_text0);


    wifiManager.addParameter(&custom_server_emoncms);
    wifiManager.addParameter(&custom_port_emoncms);
    wifiManager.addParameter(&custom_apikey_emoncms);


    WiFiManagerParameter custom_text1("<H3><center><strong> More info: Emoncms.org </H3></center></strong>");
    wifiManager.addParameter(&custom_text1);


    ///////////////////////////////// /WiFiManager //////////////////////////////////////////////////////////////////////////////////////
    //Local intialization. Once its business is done, there is no need to keep it around

    //reset settings - for testing
    //wifiManager.resetSettings();

    //sets timeout until configuration portal gets turned off
    //useful to make it all retry or go to sleep
    //in seconds
    wifiManager.setTimeout(240);

    //it starts an access point with the specified name
    //here  "AutoConnectAP"
    //and goes into a blocking loop awaiting configuration

    //WITHOUT THIS THE AP DOES NOT SEEM TO WORK PROPERLY WITH SDK 1.5 , update to at least 1.5.1
    //WiFi.mode(WIFI_STA);


    //////////////  Here you create the network for the configuration  ////////////////////

    //we add to the SSID the unique and unique ID of each ESP

    String ssid = String(ssid_apmode) + String(ESP.getChipId());

    if (!wifiManager.startConfigPortal( ssid.c_str(), "password" )) {


      Serial.println("failed to connect and hit timeout");
      delay(3000);
      //reset and try again, or maybe put it to deep sleep
      ESP.reset();
      delay(5000);
    }



    strcpy(char_server_emoncms, custom_server_emoncms.getValue());
    strcpy(char_port_emoncms, custom_port_emoncms.getValue());
    strcpy(char_apikey_emoncms, custom_apikey_emoncms.getValue());

  }

  //////////////////////////////end button configuration WIFI/////////////////////////////////////////////
  //////////////////////////////////////////////////////////////////////////


  ///////////////////// Save Parameters JSON ///////////////////
  ///////////////////////////////////////////////////////////////////////////
  //save the custom parameters to FS
  if (shouldSaveConfig) {
    Serial.println("saving config");
    DynamicJsonBuffer jsonBuffer;
    JsonObject& json = jsonBuffer.createObject();

    json["char_server_emoncms"] = char_server_emoncms;
    json["char_port_emoncms"] = char_port_emoncms;
    json["char_apikey_emoncms"] = char_apikey_emoncms;


    File configFile = SPIFFS.open("/config.json", "w");
    if (!configFile) {
      Serial.println("failed to open config file for writing");
    }

    json.printTo(Serial);
    json.printTo(configFile);
    configFile.close();
    //end save
  }
  //////////////////////////////////////////////////////////////////////////////////////
  /////////////////////////////////////////////////////////////////////////////////////

  lcd.setCursor(3, 1);
  lcd.print("Connected to...");
  delay(2000);
  lcd.setCursor(1, 2);
  lcd.print(WiFi.SSID());
  delay(5000);
  lcd.clear();

  ////////// Connection Verification ///////////////////////
  if (WiFi.SSID() == "") {

    Serial.println("Not configured !!! or do not connect to the NETWORK !!!");
    initialConfig = true;

  } else
  {

    Serial.println("Wifi Configuration Correct");

    //digitalWrite(pin_led_Wifi, LOW);

    Serial.println("Parameters");

    ///// debug serial ///////////////////
    Serial.println(char_server_emoncms);
    Serial.println(char_port_emoncms);
    //   Serial.println(char_apikey_emoncms);
    // delay(2000);

    //////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
    //// convert parameters to variables for connection to emoncms ////
    host = char_server_emoncms;
    port =  String (char_port_emoncms).toInt();
    apikey = char_apikey_emoncms;

    ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


    /////// operation that validates that there are no char * empty parameters !!! ///////

    if ( String(host).length()  ||  String(apikey).length() /*||  String(inputname1).length()*/ ) {

      parameterOK = true;

    }
    else {

      Serial.println("empty parameter fields");
      parameterOK = false;

    }

  }
  //////////////////////end - fin (void setup) ////////////////////////////////////////
}
